de = {
    "times": {
        "min": "Minute",
        "sec": "Sekunde",
        "hour": "Stunde",
        "day": "Tag",
    },
    "bot": {
        "error": "Der folgende Fehler ist aufgetreten",
        "cooldown": "Versuche es {} erneut.",
        "no_perm_title": "Fehlende Berechtigung",
        "no_perm_desc": "Mir fehlen die folgenden Berechtigungen, um diesen Befehl auszuführen.",
    },
}

en = {
    "times": {
        "min": "minute",
        "sec": "second",
        "hour": "hour",
        "day": "day",
    },
    "bot": {
        "error": "The following error occurred",
        "cooldown": "Try again {}.",
        "no_perm_title": "Missing permission",
        "no_perm_desc": "I'm missing the following permissions to execute this command.",
    },
}
